#include "main.h"
#include "events.h"
#include "engine_setup.h"
#include "scenes.h"
#include "objects.h"

GLint resx = 800;
GLint resy = 600;
GLboolean fullscreen = 1;
GLuint texture[NUM_TEXTURES]; /* Textur-Array: Speichert alle Texturen */
GLint sound_channels[NUM_SOUNDS]; /* stores which channel sdl_mixer uses for a specific sound */
Mix_Chunk *sound[NUM_SOUNDS]; /* sound-array: stores all sounds */
Mix_Music *music;
GLfloat FrameInterval; /* Enthlt die Zeit, die fr den letzten Frame gebraucht wurde (Wichtig fr Animationen!) */
GLint fps; /* Speichert die FPS */
GLuint DLIST_skybox;
GLuint DLIST_ground;
GLuint DLIST_block;
GLuint DLIST_acid;
GLuint DLIST_foot;
GLuint DLIST_glasses;
GLuint DLIST_body;
GLuint DLIST_atom;
GLuint DLIST_acidfog;
GLuint dlist[NUM_DLISTS]; /* Displaylist-Array: Speichert alle Objekte */
GLuint loop, loop2;	/* allgemeine Loopvariablen */
GLboolean scene_ende; /* Zum verlassen der Loopschleifen */

/* Speichert die Zustnde der Tasten */
GLboolean key_up;
GLboolean key_down;
GLboolean key_right;
GLboolean key_left;
GLboolean key_return;

/* Kameraposition: */
GLfloat camx;
GLfloat camz;

GLboolean intro_running; /* needed for the intro, should be put into scenes.c, when the ESCAPE event is rewritten */

GLboolean levelrun; /* Speichert ob gerade ein Spiel luft, wichtig fr das Hauptmen */

GLbyte lvl[LEVEL_SIZE_x][LEVEL_SIZE_y]; /* Level-Array, speichert den Levelaufbau */
GLboolean level_created; /* needed for the "Replay level" option */
GLuint font;	/* Speichert die Displaylists fr die Schrift */
char pre_string[63]; /* Um Integers als String auszugeben (temporre Zwischenspeicherung) */

void quit(GLint code)
{	
	for(loop = 0; loop < NUM_SOUNDS; ++loop)
	{
		Mix_FreeChunk(sound[loop]);
	}
	
	Mix_FreeMusic(music);
	music=NULL;
	
	Mix_CloseAudio();
	
	glDeleteLists(font, 256+NUM_DLISTS); /* Schrift & Objekte lschen und Speicher freigeben */
	for(loop = 0; loop < NUM_TEXTURES; ++loop) /* Alle Texturen freigeben */
	{
		glDeleteTextures( NUM_TEXTURES, &texture[loop] );
	}
	
	SDL_ShowCursor(SDL_ENABLE); /* Mauscursor wieder sichtbar machen */
	
	SDL_Quit();

	/* Programm Beenden */
	exit(code);
}

void process_args(int argc, char** argv)
{
    GLbyte argstr[255]={0};

	/* see which arguments are given: */
    for(loop = 1; loop < argc; ++loop)
    {
		strcpy(argstr, argv[loop]);
				
        switch( *argstr )
        {
            case '/':
            case '-':
				switch(argstr[1])
				{
					case 'f':
						if (loop+1 != argc)
						{
							++loop;
							switch(argv[loop][0])
							{
								case '0':
									fullscreen = 0;
									break;
								case '1':
									fullscreen = 1;
									break;
								default:
									printf("Please select if you want fullscreen or not.\nSee \"atomorun -h\" for more help\n");
									quit(-1);
									break;
							}
						}
						else
						{
							printf("Please select a resolution.\nSee \"atomorun -h\" for more help\n");
							quit(-1);
						}
						break;
					case 'r':
						if (loop+1 != argc)
						{
							++loop;
							switch(argv[loop][0])
							{
								case '1':
									resx=320;
									resy=240;
									break;
								case '2':
									resx=640;
									resy=480;
									break;
								case '3':
									resx=800;
									resy=600;
									break;
								case '4':
									resx=1024;
									resy=768;
									break;
								case '5':
									resx=1280;
									resy=1024;
									break;
								case '6':
									resx=1600;
									resy=1200;
									break;
								default:
									printf("Please select a resolution.\nSee \"atomorun -h\" for more help\n");
									quit(-1);
								break;
							}
						}	
						else
						{
							printf("Please select a resolution.\nSee \"atomorun -h\" for more help\n");
							quit(-1);
						}						
						break;
					case 'v':
						printf("%s\n", VERSION);
						quit(0);
						break;
					case 'h':
						printf("Atomorun %s help:\n", VERSION);
						printf("--------------\n");
						printf("Usage: atomorun [OPTION]...\n\n");
						printf("  -r X\tSelect resolution. X must be one of the following:\n");
						printf("\t\t1 - 320x240 (very ugly...)\n");
						printf("\t\t2 - 640x480\n");
						printf("\t\t3 - 800x600 (default)\n");
						printf("\t\t4 - 1024x768\n");
						printf("\t\t5 - 1280x1024\n");
						printf("\t\t6 - 1600x1200\n");
						printf("  -f X\tSelect fullscreen mode. X must be one of the following:\n");
						printf("\t\t0 - No fullscreen\n");
						printf("\t\t1 - Fullscreen (default)\n");
						printf("  -v\tPrint Atom'o'run version and exit\n");
						printf("Report bugs to <atomorun@whosme.de>\n");
						printf("Visit http://atomorun.whosme.de\n");
						quit(0);
						break;
					
					default:
						printf("Unknown argument %s\nSee \"atomorun -h\" for more help\n", argstr);
						quit(-1);
						break;
				}
				break;
			default:
				printf("Unknown argument %s\nSee \"atomorun -h\" for more help\n", argstr);
				quit(-1);
                break;
		}
    }
}

int main(int argc, char** argv)
{	
	level_created = GL_FALSE;
	levelrun = GL_FALSE;
	
	process_args(argc, argv);
	
	set_default(); /* Standardwerte festlegen: */
	
	setup_engine(resx, resy, fullscreen); /* Fenster erstellen, OpenGL initialisieren, usw */
	
	while(1)
	{
		/*Das Hauptmen: */
		intro_running = GL_FALSE;
		menu();
			stop_all_sounds();
		if (levelrun) level();
	}
	
	/* Dieser Punkt wird NIEMALS erreicht werden */
	return 0;
}
